/**
 * @typedef {Object} Endpoint
 */
/**
 * Base interceptor. The base class for all the interceptors
 * Calls configure() and reset().
 * @param {Object?} config The interceptor configuration object.
 */
export class Interceptor {
    /**
     * Meta description of the configuration
     * @return {Object}
     */
    static get configurationAttributes(): any;
    constructor(config: any);
    /**
     * The instance method returning the type.
     * Defaults to the constructors name (class name)
     * @return {string}
     */
    get type(): string;
    /**
     * Meta description of the configuration.
     * @return {Object}
     */
    get configurationAttributes(): any;
    /**
     * Takes attribute values from config parameters
     * and copies them over to the object.
     * Copying is done according to configurationAttributes.
     * Which means we loop over all configuration attributes
     * and then for each attribute decide if we use the default, call
     * a setter function or simply assign the attribute value.
     * @param {Object?} config
     */
    configure(config: any | null): void;
    toString(): string;
    toJSON(): any;
    /**
     * Deliver the json representation.
     * @return {Object} json representation
     */
    toJSONWithOptions(options: any): any;
    /**
     * Forget all accumulated information.
     */
    reset(): void;
    /**
     * The receive method. This method receives the request from the leading interceptor
     * and calls the trailing interceptor.
     * @param {Endpoint} endpoint
     * @param {Function} next
     * @param {any[]} args the request from the leading interceptor
     * @return {Promise<any>}
     */
    receive(endpoint: Endpoint, next: Function, ...args: any[]): Promise<any>;
}
export type Endpoint = any;
